"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const path = require("path");
const fs = require("fs");
const https = require("https");
const crypto = require("crypto");
const execa = require("execa");
const semver = require("semver");
const lang_runner_1 = require("../lib/lang-runner");
const runtimeJarVersion = '1.0.3';
const runtimeJarName = `sf-fx-runtime-java-runtime-${runtimeJarVersion}-jar-with-dependencies.jar`;
const runtimeJarUrl = `https://repo1.maven.org/maven2/com/salesforce/functions/sf-fx-runtime-java-runtime/${runtimeJarVersion}/${runtimeJarName}`;
const runtimeJarDir = path.resolve(os.tmpdir(), 'sf-fx-runtime-java');
const runtimeJarPath = path.resolve(runtimeJarDir, `sf-fx-runtime-java-${runtimeJarVersion}.jar`);
const runtimeJarSha = '1db6d78bdbb7aff7ebe011565190ca9dd4d3e68730e206628230d480d057fe1e';
class LangRunnerJava extends lang_runner_1.default {
    async detect() {
        const pomXmlPath = path.resolve(this.path, 'pom.xml');
        try {
            await fs.promises.access(pomXmlPath);
        }
        catch (_a) {
            return false;
        }
        return true;
    }
    async build() {
        await this.checkJava();
        await this.ensureRuntimeJar();
        await this.runMavenCompile();
    }
    async start() {
        await this.runRuntimeJarServe();
    }
    async checkJava() {
        let stderr = '';
        try {
            ({ stderr } = await execa.command('java -fullversion'));
        }
        catch (error) {
            throw new Error('Java executable not found.');
        }
        stderr = stderr.trim();
        const re = /"(\d+.\d+.\d+).*"/;
        const matches = re.exec(stderr);
        if (!matches || matches.length < 2) {
            throw new Error(`Could not determine Java version from: ${stderr}`);
        }
        this.javaVersion = matches[1];
    }
    async ensureRuntimeJar() {
        if (await this.checkRuntimeJar()) {
            return;
        }
        await this.downloadRuntimeJar();
    }
    async runMavenCompile() {
        let mvnBin = path.resolve(this.path, 'mvnw');
        try {
            // check to see if `mvnw` exists in function root.
            await fs.promises.access(mvnBin);
        }
        catch (_a) {
            // mvnw does not exist, use user installed `mvn` instead.
            mvnBin = 'mvn';
        }
        try {
            // ensure mvn or mvnw exists and is executable
            await execa.command(`${mvnBin} --version`, { cwd: this.path });
        }
        catch (error) {
            throw new Error(`Could not run maven executable: ${error}`);
        }
        try {
            await execa.command(`${mvnBin} compile`, { stdio: 'inherit', cwd: this.path });
        }
        catch (error) {
            throw new Error(`Could not compile function with maven: ${error}`);
        }
    }
    async checkRuntimeJar() {
        try {
            await fs.promises.access(runtimeJarPath);
        }
        catch (_a) {
            return false;
        }
        return true;
    }
    async downloadRuntimeJar() {
        try {
            await this.ensureDirExists(runtimeJarDir);
            await this.downloadToFile(runtimeJarUrl, runtimeJarPath);
            await this.verifyFileSha(runtimeJarPath, runtimeJarSha);
        }
        catch (err) {
            try {
                await fs.promises.unlink(runtimeJarPath);
            }
            catch (_a) {
                // failed to delete the jar -- probably never created it.
            }
            throw err;
        }
    }
    async runRuntimeJarServe() {
        let agentLibArg = '-agentlib:jdwp=transport=dt_socket,server=y,suspend=n,address=';
        if (this.javaVersion && semver.satisfies(this.javaVersion, '1.8.*')) {
            agentLibArg = agentLibArg + this.debugPort;
        }
        else {
            agentLibArg = agentLibArg + `*:${this.debugPort}`;
        }
        await execa.command(`java ${agentLibArg} -jar ${runtimeJarPath} serve --host=${this.host} --port=${this.port} ${this.path}`, {
            cwd: this.path,
            stdio: 'inherit',
        });
    }
    downloadToFile(url, path) {
        return new Promise((resolve, reject) => {
            const file = fs.createWriteStream(path);
            https
                .get(url, (resp) => {
                const headers = JSON.stringify(resp.headers);
                if (resp.statusCode !== 200) {
                    reject(`Unexpected status code: ${resp.statusCode}`);
                    return;
                }
                resp.pipe(file);
                file.on('finish', resolve);
                file.on('error', reject);
            })
                .on('error', (err) => {
                reject(err);
            });
        });
    }
    async verifyFileSha(path, expectedDigest) {
        const shaHash = crypto.createHash('sha256');
        const fileBuf = await fs.promises.readFile(path);
        shaHash.update(fileBuf);
        const actualDigest = shaHash.digest('hex');
        if (expectedDigest !== actualDigest) {
            throw new Error(`${path} could not be validated.`);
        }
    }
    async ensureDirExists(dir) {
        try {
            await fs.promises.mkdir(dir);
        }
        catch (err) {
            // Don't throw if the dir already existed
            if (err.code !== 'EEXIST') {
                throw err;
            }
        }
    }
}
exports.default = LangRunnerJava;
