"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const execa = require("execa");
const semver = require("semver");
const lang_runner_1 = require("../lib/lang-runner");
class NodeJsLangRunner extends lang_runner_1.default {
    async detect() {
        const packageJsonPath = path.resolve(this.path, 'package.json');
        try {
            await fs.promises.access(packageJsonPath);
        }
        catch (_a) {
            return false;
        }
        return true;
    }
    async build() {
        await this.checkNodeJs();
        await this.checkNpx();
        await this.runNpmBuild();
    }
    async start() {
        await this.startRuntime();
    }
    async checkNodeJs() {
        let stdout = '';
        try {
            const cmd = await execa.command('node -v');
            ({ stdout } = cmd);
        }
        catch (error) {
            throw new Error('Node.JS executable not found.');
        }
        const version = semver.clean(stdout);
        if (!version || semver.lt(version, '14.0.0')) {
            throw new Error('Node.js functions require Node.js 14 or greater.');
        }
    }
    async checkNpx() {
        try {
            await execa('npx', ['-v']);
        }
        catch (error) {
            throw new Error('npx executable not found.');
        }
    }
    async runNpmBuild() {
        var _a;
        const packageJsonPath = path.resolve(this.path, 'package.json');
        let packageJsonContent;
        try {
            packageJsonContent = await fs.promises.readFile(packageJsonPath, 'utf8');
        }
        catch (_b) {
            throw new Error(`Could not read 'package.json' from ${packageJsonPath}.`);
        }
        let packageJson;
        try {
            packageJson = JSON.parse(packageJsonContent);
        }
        catch (_c) {
            throw new Error(`Cound not parse 'package.json' as JSON from ${packageJsonPath}.`);
        }
        if ((_a = packageJson.scripts) === null || _a === void 0 ? void 0 : _a.build) {
            try {
                await execa.command(['npm', 'run', 'build', '--prefix', this.path].join(' '), { stdio: 'inherit' });
            }
            catch (err) {
                throw new Error(`Could not execute npm build script: ${err}`);
            }
        }
    }
    async startRuntime() {
        try {
            await execa.command([
                'npx',
                '-y',
                '-g',
                '-p',
                '@heroku/sf-fx-runtime-nodejs@0.9.1',
                'sf-fx-runtime-nodejs',
                'serve',
                this.path,
                '--host',
                this.host,
                '--port',
                this.port.toString(),
                '--debug-port',
                this.debugPort.toString(),
            ].join(' '), {
                stdio: 'inherit',
            });
        }
        catch (err) {
            throw new Error(`Could not execute function runtime: ${err}`);
        }
    }
}
exports.default = NodeJsLangRunner;
